//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Diagramming.Layout;


namespace MindFusion.Diagramming.WinForms.Samples.CS.CompositeLayout
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			diagram.DefaultShape = Shapes.Ellipse;
			diagram.LinkHeadShape = ArrowHeads.PointerArrow;
			diagram.LinkHeadShapeSize = 4;
            diagram.DiagramLinkStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
			diagram.ShadowsStyle = ShadowsStyle.None;
            diagram.ShapeNodeStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(156, 170, 198));
            diagram.ShapeNodeStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));

            diagram.DiagramLinkStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));
            diagram.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
            diagram.DiagramLinkStyle.HeadStroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));

		}

		private void button2_Click(object sender, EventArgs e)
		{
			// Reset styles
			foreach (DiagramNode node in diagram.Nodes)
			{
                node.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(156, 170, 198));
                node.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(97, 106, 127), 0);
			}

			foreach (DiagramLink link in diagram.Links)
			{
                link.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));
                link.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192), 0);
                link.HeadPen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192), 0);
			}

			MindFusion.Diagramming.Layout.CompositeLayout layout =
				new MindFusion.Diagramming.Layout.CompositeLayout();

			layout.RotateSubgraphs = checkBox2.Checked;
			layout.EmphasizeTrees = checkBox1.Checked;
			if (radioButton1.Checked)
				layout.PartitionMethod = MindFusion.Layout.PartitionMethod.BiconnectedComponents;
			else if (radioButton2.Checked)
				layout.PartitionMethod = MindFusion.Layout.PartitionMethod.ShortestPath;
			else if (radioButton3.Checked)
				layout.PartitionMethod = MindFusion.Layout.PartitionMethod.Custom;

			if (checkBox4.Checked)
			{
				layout.UseAdditionalPartitioning = true;
				layout.BendLinks = true;
			}

			TreeLayout subtreeLayout = new TreeLayout();
			subtreeLayout.Type = TreeLayoutType.Radial;
			subtreeLayout.IgnoreLinkDirection = true;
			subtreeLayout.LayoutNode = OnSubtreeLayoutNode;
			subtreeLayout.LayoutLink = OnSubtreeLayoutLink;
			layout.SubtreeLayout = subtreeLayout;

			if (radioButton4.Checked)
			{
				CircularLayout subgraphLayout = new CircularLayout();
				subgraphLayout.Radius = 0;
				subgraphLayout.SplitGraph = false;
				subgraphLayout.LayoutNode = OnSubgraphLayoutNode;
				subgraphLayout.LayoutLink = OnSubgraphLayoutLink;
				layout.SubgraphLayout = subgraphLayout;
			}
			else if (radioButton5.Checked)
			{
				//LayeredLayout subgraphLayout = new LayeredLayout();
				//subgraphLayout.Orientation = MindFusion.Diagramming.Layout.Orientation.Vertical;
				//subgraphLayout.IgnoreNodeSize = false;
				//subgraphLayout.StraightenLongLinks = false;
				//layout.CustomLayout = subgraphLayout;
				//layout.SubtreeLayout = null;
				//layout.SubgraphLayout = null;
				layout.CustomLayout = OnCustomLayeredLayout;
			}
			else if (radioButton6.Checked)
			{
				//AnnealLayout subgraphLayout = new AnnealLayout();
				//layout.CustomLayout = subgraphLayout;
				//layout.SubtreeLayout = null;
				//layout.SubgraphLayout = null;
				layout.CustomLayout = OnCustomAnnealLayout;
			}
			else if (radioButton7.Checked)
			{
				//OrthogonalLayout subgraphLayout = new OrthogonalLayout();
				//layout.CustomLayout = subgraphLayout;
				//layout.SubtreeLayout = null;
				//layout.SubgraphLayout = null;
				layout.CustomLayout = OnCustomOrthogonalLayout;
			}

			AnnealLayout masterLayout = new AnnealLayout();
			masterLayout.CrossingLinksCost = 0;
			masterLayout.LayoutNode = OnMasterLayoutNode;
			masterLayout.LayoutLink = OnMasterLayoutLink;
			layout.MasterLayout = masterLayout;

			List<DiagramNodeCollection> partition = new List<DiagramNodeCollection>();
			if (radioButton3.Checked)
			{
				DiagramNodeCollection currentPartition = null;
				for (int i = 0; i < diagram.Nodes.Count; i++)
				{
					if (i % 50 == 0)
					{
						currentPartition = new DiagramNodeCollection();
						partition.Add(currentPartition);
					}

					currentPartition.Add(diagram.Nodes[i]);
				}

				layout.CustomPartition = partition;
			}

			if (checkBox3.Checked && radioButton3.Checked)
			{
				Color[] colors = new Color[]
				{
					Color.Red,
					Color.Yellow,
					Color.Green,
					Color.Blue,
					Color.Orange,
					Color.Purple,
					Color.LightSteelBlue,
					Color.Magenta,
					Color.Black,
					Color.Turquoise,
				};

				int p = 0;
				foreach (DiagramNodeCollection nodes in partition)
				{
					foreach (DiagramNode node in nodes)
					{
						node.Brush = new MindFusion.Drawing.SolidBrush(colors[p % colors.Length]);
						node.Pen = new MindFusion.Drawing.Pen(colors[p % colors.Length], 0);
					}

					p++;
				}
			}

			int size = 32;
			int temp;
			if (int.TryParse(textBox1.Text, out temp))
				size = temp;
			layout.DesiredSubgraphSize = size;

			layout.Arrange(diagram);

			diagram.ResizeToFitItems(5);
		}

		private bool OnCustomLayeredLayout(Diagram diagram, DiagramItemCollection items)
		{
			LayeredLayout subgraphLayout = new LayeredLayout();
			subgraphLayout.Orientation = MindFusion.Diagramming.Layout.Orientation.Vertical;
			subgraphLayout.IgnoreNodeSize = false;
			subgraphLayout.StraightenLongLinks = false;
			return subgraphLayout.Arrange(diagram, items);
		}

		private bool OnCustomAnnealLayout(Diagram diagram, DiagramItemCollection items)
		{
			AnnealLayout subgraphLayout = new AnnealLayout();
			return subgraphLayout.Arrange(diagram, items);
		}

		private bool OnCustomOrthogonalLayout(Diagram diagram, DiagramItemCollection items)
		{
			OrthogonalLayout subgraphLayout = new OrthogonalLayout();
			return subgraphLayout.Arrange(diagram, items);
		}


		private void OnSubtreeLayoutNode(DiagramNode node, RectangleF oldBounds)
		{
		}

		private void OnSubtreeLayoutLink(DiagramLink link)
		{
			if (radioButton3.Checked && checkBox3.Checked)
			{
				link.Pen = new MindFusion.Drawing.Pen(link.Origin.Brush, 0);
				link.HeadPen = new MindFusion.Drawing.Pen(link.Origin.Brush, 0);
				link.Brush = link.Origin.Brush;
			}
		}

		private void OnSubgraphLayoutNode(DiagramNode node, RectangleF oldBounds)
		{
		}

		private void OnSubgraphLayoutLink(DiagramLink link)
		{
			if (radioButton3.Checked && checkBox3.Checked)
			{
				link.Pen = new MindFusion.Drawing.Pen(link.Origin.Brush, 0);
				link.HeadPen = new MindFusion.Drawing.Pen(link.Origin.Brush, 0);
				link.Brush = link.Origin.Brush;
			}
		}

		private void OnMasterLayoutNode(DiagramNode node, RectangleF oldBounds)
		{
			node.Pen = new MindFusion.Drawing.Pen(Color.Green, 0);
		}

		private void OnMasterLayoutLink(DiagramLink link)
		{
			link.Pen = new MindFusion.Drawing.Pen(Color.Green, 0);
		}

		private void clearToolStripMenuItem_Click(object sender, EventArgs e)
		{
			diagram.ClearAll();
		}

		private void openToolStripMenuItem_Click(object sender, EventArgs e)
		{
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Filter = "*.xml|*.xml|*.*|*.*||";
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				diagram.LoadFromXml(dialog.FileName);
			}
		}

		private void saveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog dialog = new SaveFileDialog();
			dialog.Filter = "*.xml|*.xml|*.*|*.*||";
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				diagram.SaveToXml(dialog.FileName);
			}
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void CreateBlock(int numNodes, int extraLinks, BlockFrame frame, string label)
		{
			CreateBlock(numNodes, extraLinks, frame, label, 0);
		}

		private void CreateBlock(int numNodes, int extraLinks, BlockFrame frame, string label, int extraNodes)
		{
			DiagramNodeCollection nodes = diagram.Nodes;
			int startIndex = nodes.Count;

			for (int n = 0; n < numNodes; ++n)
			{
				ShapeNode node = diagram.Factory.CreateShapeNode(0, 0, 10, 10);
				node.Text = label;
			}

			if (frame == BlockFrame.Cycle)
			{
				for (int l = 0; l < numNodes; ++l)
					diagram.Factory.CreateDiagramLink(nodes[startIndex + l], nodes[startIndex + (l + 1) % numNodes]);
			}
			if (frame == BlockFrame.Forest)
			{
				for (int l = startIndex + 1; l < startIndex + numNodes; ++l)
					diagram.Factory.CreateDiagramLink(nodes[l], nodes[(l - 1) / 10 * 10]);
			}
			if (frame == BlockFrame.Star)
			{
				for (int l = startIndex; l < startIndex + numNodes; ++l)
					if (l != startIndex + numNodes / 2)
						diagram.Factory.CreateDiagramLink(nodes[l], nodes[startIndex + numNodes / 2]);
			}

			Random random = new Random(1);
			for (int l = 1; l < extraLinks; ++l)
			{
				int s = random.Next(startIndex, startIndex + numNodes);
				int t = random.Next(startIndex, startIndex + numNodes);
				while (s == t || Connected(nodes[s], nodes[t]))
					t = random.Next(startIndex, startIndex + numNodes);
				diagram.Factory.CreateDiagramLink(nodes[s], nodes[t]);
			}

			for (int n = 0; n < extraNodes; ++n)
			{
				ShapeNode node = diagram.Factory.CreateShapeNode(0, 0, 10, 10);
				node.Text = label;

				int s = random.Next(startIndex, diagram.Nodes.Count - 1);
				int t = random.Next(startIndex, diagram.Nodes.Count - 1);
				while (s == t)
					t = random.Next(startIndex, diagram.Nodes.Count - 1);
				diagram.Factory.CreateDiagramLink(node, nodes[s]);
				diagram.Factory.CreateDiagramLink(node, nodes[t]);
			}
		}

		bool Connected(DiagramNode node1, DiagramNode node2)
		{
			return GetCommonLinks(node1, node2).Count > 0;
		}

		DiagramLinkCollection GetCommonLinks(DiagramNode node1, DiagramNode node2)
		{
			DiagramLinkCollection commonLinks = new DiagramLinkCollection();

			foreach (DiagramLink link in node1.OutgoingLinks)
				if (link.Destination == node2)
					commonLinks.Add(link);

			foreach (DiagramLink link in node1.IncomingLinks)
				if (link.Origin == node2)
					commonLinks.Add(link);

			return commonLinks;
		}

		private void button1_Click(object sender, EventArgs e)
		{
			diagram.ClearAll();

			int nodes = 1000;
			int links = 100;

			int temp = 0;
			if (int.TryParse(textBox2.Text, out temp))
			{
				if (temp > 0)
					nodes = temp;
			}
			if (int.TryParse(textBox3.Text, out temp))
			{
				if (temp > 0)
					links = temp;
			}

			CreateBlock(nodes, links, BlockFrame.Forest, "");
		}

		private void radioButton3_CheckedChanged(object sender, EventArgs e)
		{
			checkBox3.Enabled = radioButton3.Checked;
		}

		private void zoomInToolStripMenuItem_Click(object sender, EventArgs e)
		{
			diagramView.ZoomIn();
		}

		private void zoomOutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			diagramView.ZoomOut();
		}

		private void fitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			diagramView.ZoomToFit();
		}

		private void noZoomToolStripMenuItem_Click(object sender, EventArgs e)
		{
			diagramView.ZoomFactor = 100;
		}
	}

	enum BlockFrame
	{
		Cycle,
		Forest,
		Star
	}
}
